## $Id: browser.py 9783 2012-12-07 08:27:27Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import grok
from zope.component import getUtility
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent
    )
from waeup.kofa.browser.layout import KofaPage
from waeup.kofa.interfaces import IKofaUtils
from waeup.kofa.utils.helpers import to_timezone
from waeup.uniben.students.interfaces import ICustomStudentOnlinePayment
from waeup.uniben.applicants.interfaces import ICustomApplicantOnlinePayment
from waeup.uniben.interfaces import MessageFactory as _

PRODUCT_ID = '57'
SITE_NAME = 'uniben-kofa.waeup.org'
PROVIDER_ACCT = '1010764827'
PROVIDER_BANK_ID = '117'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'Uniben'
CURRENCY = '566'
GATEWAY_AMT = 150.0
#QUERY_URL = 'https://webpay.interswitchng.com/paydirect/services/TransactionQueryURL.aspx'
#QUERY_URL = 'https://testwebpay.interswitchng.com/test_paydirect/services/TransactionQueryURL.aspx'
POST_ACTION = 'https://webpay.interswitchng.com/paydirect/webpay/pay.aspx'
#POST_ACTION = 'https://testwebpay.interswitchng.com/test_paydirect/webpay/pay.aspx'

HOST = 'webpay.interswitchng.com'
#HOST = 'testwebpay.interswitchng.com'
URL = '/paydirect/services/TransactionQueryWs.asmx'
#URL = '/test_paydirect/services/TransactionQueryWs.asmx'
httplib.HTTPConnection.debuglevel = 0

class InterswitchPageStudent(KofaPage):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    grok.name('goto_interswitch')
    grok.template('student_goto_interswitch')
    grok.require('waeup.payStudent')
    label = _('Submit data to CollegePAY (Interswitch Payment Gateway)')
    submit_button = _('Submit')
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID

    def update(self):
        #if self.context.p_state != 'unpaid':
        if self.context.p_state == 'paid':
            self.flash(_("Payment ticket can't be re-send to CollegePAY."))
            self.redirect(self.url(self.context, '@@index'))
            return

        student = self.student = self.context.student
        certificate = getattr(student['studycourse'],'certificate',None)
        self.amount_auth = 100 * self.context.amount_auth
        xmldict = {}
        if certificate is not None:
            xmldict['department'] = certificate.__parent__.__parent__.code
            xmldict['faculty'] = certificate.__parent__.__parent__.__parent__.code
        else:
            xmldict['department'] = None
            xmldict['faculty'] = None
        self.category = getUtility(IKofaUtils).PAYMENT_CATEGORIES[self.context.p_category]
        tz = getUtility(IKofaUtils).tzinfo
        self.local_date_time = to_timezone(
            self.context.creation_date, tz).strftime("%Y-%m-%d %H:%M:%S %Z")
        self.site_redirect_url = self.url(self.context, 'request_webservice')
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        xmldict['institution_acct'] = '000000000000'
        xmldict['institution_bank_id'] = '00'
        xmldict['institution_amt'] = '0.0'
        if self.context.p_category == 'schoolfee':
            provider_amt = 1500.0
            if student.current_mode.endswith('_ft'):
                self.pay_item_id = '5700'
                if student.current_mode in ('ug_ft','de_ft','ct_ft','ume_ft'):
                    xmldict['institution_acct'] = '2017506430'
                    xmldict['institution_bank_id'] = '8'
                elif student.current_mode in ('dp_ft'):
                    xmldict['institution_acct'] = '9201805071'
                    xmldict['institution_bank_id'] = '17'
                elif student.current_mode in ('pg_ft'):
                    xmldict['institution_acct'] = '5330832799'
                    xmldict['institution_bank_id'] = '51'
            elif student.current_mode.endswith('_pt'):
                self.pay_item_id = '5701'
                if student.current_mode in ('ug_pt','de_pt','ct_pt'):
                    xmldict['institution_acct'] = '0122009929'
                    xmldict['institution_bank_id'] = '16'
                elif student.current_mode in ('dp_pt'):
                    xmldict['institution_acct'] = '9201805071'
                    xmldict['institution_bank_id'] = '17'
                elif student.current_mode in ('pg_pt'):
                    xmldict['institution_acct'] = '0031716047'
                    xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'clearance':
            self.pay_item_id = '5702'
            provider_amt = 1500.0
            if student.current_mode == 'pg_ft':
                xmldict['institution_acct'] = '5330832799'
                xmldict['institution_bank_id'] = '51'
            elif student.current_mode == 'pg_pt':
                xmldict['institution_acct'] = '0031716047'
                xmldict['institution_bank_id'] = '10'
            elif student.current_mode == 'dp_pt':
                xmldict['institution_acct'] = '9201805071'
                xmldict['institution_bank_id'] = '17'
            else:
                xmldict['institution_bank_id'] = '7'
                xmldict['institution_acct'] = '1003475516'
        elif self.context.p_category == 'gown':
            self.pay_item_id = '5704'
            provider_amt = 0.0
            xmldict['institution_bank_id'] = '7'
            xmldict['institution_acct'] = '1016232382'
        elif self.context.p_category.startswith('hostel_maintenance'):
            self.pay_item_id = '5705'
            provider_amt = 0.0
            xmldict['institution_bank_id'] = '129'
            xmldict['institution_acct'] = '0014414547'

        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['institution_item_name'] = self.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 100 * (
            self.context.amount_auth - provider_amt - GATEWAY_AMT)
        # Interswitch amount is not part of the xml data
        if provider_amt == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        return

class InterswitchPageApplicant(KofaPage):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    grok.require('waeup.payApplicant')
    grok.template('applicant_goto_interswitch')
    grok.name('goto_interswitch')
    label = _('Submit data to CollegePAY (Interswitch Payment Gateway)')
    submit_button = _('Submit')
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = '5703'
    product_id = PRODUCT_ID

    def update(self):
        if self.context.p_state != 'unpaid':
            self.flash(_("Payment ticket can't be re-send to CollegePAY."))
            self.redirect(self.url(self.context, '@@index'))
            return
        if self.context.__parent__.__parent__.expired \
            and self.context.__parent__.__parent__.strict_deadline:
            self.flash(_("Payment ticket can't be send to CollegePAY. "
                         "Application period has expired."))
            self.redirect(self.url(self.context, '@@index'))
            return
        self.applicant = self.context.__parent__
        self.amount_auth = 100 * self.context.amount_auth
        xmldict = {}
        self.category = getUtility(IKofaUtils).PAYMENT_CATEGORIES[self.context.p_category]
        tz = getUtility(IKofaUtils).tzinfo
        self.local_date_time = to_timezone(
            self.context.creation_date, tz).strftime("%Y-%m-%d %H:%M:%S %Z")
        self.site_redirect_url = self.url(self.context, 'request_webservice')
        provider_amt = 400.0
        if self.applicant.applicant_id.startswith('pg'):
            xmldict['institution_acct'] = '0031716030'
            xmldict['institution_bank_id'] = '10'
        elif self.applicant.applicant_id.startswith('dp'):
            xmldict['institution_acct'] = '9201805071'
            xmldict['institution_bank_id'] = '17'
        else:
            xmldict['institution_acct'] = '6220032503'
            xmldict['institution_bank_id'] = '51'
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_amt'] = 100 * (self.context.amount_auth - provider_amt - GATEWAY_AMT)
        xmldict['institution_item_name'] = self.context.p_category
        xmldict['institution_name'] = INSTITUTION_NAME
        # Interswitch amount is not part of the xml data
        xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        return

class InterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL

class InterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL